import sxtwl

ymc = [u"十一", u"十二", u"正", u"二", u"三", u"四", u"五", u"六", u"七", u"八", u"九", u"十"]
rmc = [u"初一", u"初二", u"初三", u"初四", u"初五", u"初六", u"初七", u"初八", u"初九", u"初十",
       u"十一", u"十二", u"十三", u"十四", u"十五", u"十六", u"十七", u"十八", u"十九",
       u"二十", u"廿一", u"廿二", u"廿三", u"廿四", u"廿五", u"廿六", u"廿七", u"廿八", u"廿九", u"三十", u"卅一"]

lunar = sxtwl.Lunar()  # 实例化日历库


class CalendarTool:
    """
    万年历工具
    """

    @staticmethod
    def __lunar_to_solar__(y, m, d):
        """
        # 阴历转阳历
        :param y:年
        :param m: 月
        :param d: 日
        :return:
        """
        day = lunar.getDayByLunar(y, m, d, False)
        return '{}年{}月{}日'.format(day.y, day.m, day.d)

    @staticmethod
    def __solar_to_lunar__(y, m, d):
        """
        阳历转阴历
        :param y:
        :param m:
        :param d:
        :return:
        """
        day = lunar.getDayBySolar(y, m, d)
        return '{}年{}月{}日'.format(y, ymc[day.Lmc], rmc[day.Ldi])



# -*- coding: utf-8 -*-
from threading import Thread
import arrow
import regex as re
from smart_tag_tools.TimeTagHelper.TimeNormalizer import TimeNormalizer


class MyThread(Thread):
    def __init__(self, func, args=()):
        super(MyThread, self).__init__()
        self.func = func
        self.args = args

    def run(self):
        self.result = self.func(*self.args)

    def get_result(self):
        Thread.join(self)
        try:
            return self.result
        except:
            return None


def date_format(date_str):
    """
    日期格式化转换
    """
    _count = date_str.count('-')
    switch = {
        0: lambda x: x,
        1: lambda x: '{}年{}月'.format(*re.findall(r'(\d{4})-(\d{2})', x, re.S)[0]),
        2: lambda x: '{}年{}月{}日'.format(*re.findall(r'(\d{4})-(\d{2})-(\d{2})', x, re.S)[0])
    }
    return switch.get(_count, 0)(date_str)


def handle(time_list, base_time=arrow.now()):
    """
    时间处理
    :param time_list: 时间字符串列表
    :param base_time: 基准时间
    :return:
    """
    tn = TimeNormalizer()
    thred_list = list()
    result_list = list()
    for time_string in time_list:
        t = MyThread(tn.parse, args=(time_string, base_time))
        t.start()
        thred_list.append(t)

    for thred in thred_list:
        result = thred.get_result().get("timestamp", "")
        result_list.append(date_format(result))
    return result_list

# time_list = ["6月2号星期五农历5月初八", "星期二", "农历十一月初八", "20周年", "3号", "一年", "2019年", "70周年", "70年", "100年", "5000年"]
# print(handle(time_list))



# -*- coding: utf-8 -*-
import pickle
import regex as re
import arrow
import json
import os
import threading

from smart_tag_tools.TimeTagHelper.StringPreHandler import StringPreHandler
from smart_tag_tools.TimeTagHelper.TimePoint import TimePoint
from smart_tag_tools.TimeTagHelper.TimeUnit import TimeUnit
from smart_tag_tools.TimeTagHelper import glo


# 时间表达式识别的主要工作类
class TimeNormalizer:
    def __init__(self, isPreferFuture=False):
        self.lock = threading.Lock()
        self.isPreferFuture = isPreferFuture
        self.pattern, self.holi_solar, self.holi_lunar = self.init()

    # 这里对一些不规范的表达做转换
    def _filter(self, input_query):
        # 特殊情况的处理
        input_query = StringPreHandler.specialTranslator(input_query)
        # 这里对于下个周末这种做转化 把个给移除掉
        input_query = StringPreHandler.numberTranslator(input_query)

        rule = "[0-9]月[0-9]"
        pattern = re.compile(rule)
        match = pattern.search(input_query)
        if match != None:
            glo.set_value("is_Holiday", True)
            index = input_query.find('月')
            rule = "日|号"
            pattern = re.compile(rule)
            match = pattern.search(input_query[index:])
            if match == None:
                rule = "[0-9]月[0-9]+"
                pattern = re.compile(rule)
                match = pattern.search(input_query)
                if match != None:
                    end = match.span()[1]
                    input_query = input_query[:end] + '号' + input_query[end:]

        rule = "月"
        pattern = re.compile(rule)
        match = pattern.search(input_query)
        if match == None:
            input_query = input_query.replace('个', '')

        input_query = input_query.replace('中旬', '15号')
        input_query = input_query.replace('傍晚', '午后')
        input_query = input_query.replace('大年', '')
        input_query = input_query.replace('五一', '劳动节')
        input_query = input_query.replace('十一', '国庆节')
        input_query = input_query.replace('白天', '早上')
        input_query = input_query.replace('：', ':')
        return input_query

    def init(self):
        fpath = os.path.dirname(__file__) + '/resource/reg.pkl'
        try:
            with open(fpath, 'rb') as f:
                pattern = pickle.load(f)
        except:
            with open(os.path.dirname(__file__) + '/resource/regex.txt', 'r', encoding="utf-8") as f:
                content = f.read().replace("\n", "")
            p = re.compile(content)
            with open(fpath, 'wb') as f:
                pickle.dump(p, f)
            with open(fpath, 'rb') as f:
                pattern = pickle.load(f)
        with open(os.path.dirname(__file__) + '/resource/holi_solar.json', 'r', encoding='utf-8') as f:
            holi_solar = json.load(f)
        with open(os.path.dirname(__file__) + '/resource/holi_lunar.json', 'r', encoding='utf-8') as f:
            holi_lunar = json.load(f)
        return pattern, holi_solar, holi_lunar

    def parse(self, target, timeBase=arrow.now()):
        """
        TimeNormalizer的构造方法，timeBase取默认的系统当前时间
        :param timeBase: 基准时间点
        :param target: 待分析字符串
        :return: 时间单元数组
        """
        self.lock.acquire()
        glo.set_value("is_Holiday", False)
        self.isTimeSpan = False
        self.invalidSpan = False
        self.timeSpan = ''
        self.target = self._filter(target)
        self.timeBase = arrow.get(timeBase).format('YYYY-M-D-H-m-s')
        self.nowTime = timeBase
        self.oldTimeBase = self.timeBase
        self.__preHandling()
        self.timeToken = self.__timeEx()
        dic = {}
        res = self.timeToken

        if self.isTimeSpan:
            if self.invalidSpan:
                dic['error'] = 'no time pattern could be extracted.'
            else:
                result = {}
                dic['type'] = 'timedelta'
                dic['timedelta'] = self.timeSpan
                # print(dic['timedelta'])
                index = dic['timedelta'].find('days')

                days = int(dic['timedelta'][:index - 1])
                result['year'] = int(days / 365)
                result['month'] = int(days / 30.417 - result['year'] * 12)
                result['day'] = int(days - result['year'] * 365 - result['month'] * 30.417)
                index = dic['timedelta'].find(',')
                time = dic['timedelta'][index + 1:]
                time = time.split(':')
                result['hour'] = int(time[0])
                result['minute'] = int(time[1])
                result['second'] = int(time[2])
                dic['timedelta'] = result
        else:
            if len(res) == 0:
                dic['error'] = 'no time pattern could be extracted.'
            elif len(res) == 1:
                dic['type'] = 'timestamp'
                if glo.get_value("is_Holiday"):
                    dic['timestamp'] = res[0].time.format("YYYY-MM-DD HH:mm:ss").replace(" 00:00:00", "").replace(
                        ":00:00", "点")
                else:
                    dic['timestamp'] = res[0].time.format("YYYY-MM-DD HH:mm:ss").replace("-01-01 00:00:00",
                                                                                         "年").replace("-01 00:00:00",
                                                                                                      "").replace(
                        " 00:00:00", "").replace(":00:00", "点")
            else:
                dic['type'] = 'timespan'
                dic['timespan'] = [res[0].time.format("YYYY-MM-DD HH:mm:ss"), res[1].time.format("YYYY-MM-DD HH:mm:ss")]

        self.lock.release()
        return dic

    def __preHandling(self):
        """
        待匹配字符串的清理空白符和语气助词以及大写数字转化的预处理
        :return:
        """
        self.target = StringPreHandler.delKeyword(self.target, "\\s+")  # 清理空白符
        self.target = StringPreHandler.delKeyword(self.target, "[的]+")  # 清理语气助词
        self.target = StringPreHandler.numberTranslator(self.target)  # 大写数字转化

    def __timeEx(self):
        """

        :param target: 输入文本字符串
        :param timeBase: 输入基准时间
        :return: TimeUnit[]时间表达式类型数组
        """
        startline = -1
        endline = -1
        rpointer = 0
        temp = []

        match = self.pattern.finditer(self.target)
        for m in match:
            startline = m.start()
            if startline == endline:
                rpointer -= 1
                temp[rpointer] = temp[rpointer] + m.group()
            else:
                temp.append(m.group())
            endline = m.end()
            rpointer += 1
        res = []
        # 时间上下文： 前一个识别出来的时间会是下一个时间的上下文，用于处理：周六3点到5点这样的多个时间的识别，第二个5点应识别到是周六的。
        contextTp = TimePoint()
        # print(self.timeBase)
        # print('temp', temp)
        for i in range(0, rpointer):
            # 这里是一个类嵌套了一个类
            res.append(TimeUnit(temp[i], self, contextTp))
            # res[i].tp.tunit[3] = -1
            contextTp = res[i].tp
            # print(self.nowTime.year)
            # print(contextTp.tunit)
        res = self.__filterTimeUnit(res)

        return res

    def __filterTimeUnit(self, tu_arr):
        """
        过滤timeUnit中无用的识别词。无用识别词识别出的时间是1970.01.01 00:00:00(fastTime=0)
        :param tu_arr:
        :return:
        """
        if (tu_arr is None) or (len(tu_arr) < 1):
            return tu_arr
        res = []
        for tu in tu_arr:
            if tu.time.timestamp != 0:
                res.append(tu)
        return res



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2017/11/20 15:42
# @Author  : zhm
# @File    : StringPreHandler.py
# @Software: PyCharm
import regex as re

# * 字符串预处理模块，为分析器TimeNormalizer提供相应的字符串预处理服务
class StringPreHandler:
    @classmethod
    def delKeyword(cls, target, rules):
        """
        该方法删除一字符串中所有匹配某一规则字串
        可用于清理一个字符串中的空白符和语气助词
        :param target: 待处理字符串
        :param rules: 删除规则
        :return: 清理工作完成后的字符串
        """
        pattern = re.compile(rules)
        res = pattern.sub('', target)
        # print res
        return res


    @classmethod
    def numberTranslator(cls, target):
        """
        该方法可以将字符串中所有的用汉字表示的数字转化为用阿拉伯数字表示的数字
        如"这里有一千两百个人，六百零五个来自中国"可以转化为
        "这里有1200个人，605个来自中国"
        此外添加支持了部分不规则表达方法
        如两万零六百五可转化为20650
        两百一十四和两百十四都可以转化为214
        一六零加一五八可以转化为160+158
        该方法目前支持的正确转化范围是0-99999999
        该功能模块具有良好的复用性
        :param target: 待转化的字符串
        :return: 转化完毕后的字符串
        """
        pattern = re.compile("[一二两三四五六七八九123456789]万[一二两三四五六七八九123456789](?!(千|百|十))")
        match = pattern.finditer(target)
        for m in match:
            group = m.group()
            s = group.split("万")
            s = list(filter(None, s))
            num = 0
            if len(s) == 2:
                num += cls.wordToNumber(s[0]) * 10000 + cls.wordToNumber(s[1]) * 1000
            target = pattern.sub(str(num), target, 1)

        pattern = re.compile("[一二两三四五六七八九123456789]千[一二两三四五六七八九123456789](?!(百|十))")
        match = pattern.finditer(target)
        for m in match:
            group = m.group()
            s = group.split("千")
            s = list(filter(None, s))
            num = 0
            if len(s) == 2:
                num += cls.wordToNumber(s[0]) * 1000 + cls.wordToNumber(s[1]) * 100
            target = pattern.sub(str(num), target, 1)

        pattern = re.compile("[一二两三四五六七八九123456789]百[一二两三四五六七八九123456789](?!十)")
        match = pattern.finditer(target)
        for m in match:
            group = m.group()
            s = group.split("百")
            s = list(filter(None, s))
            num = 0
            if len(s) == 2:
                num += cls.wordToNumber(s[0]) * 100 + cls.wordToNumber(s[1]) * 10
            target = pattern.sub(str(num), target, 1)

        pattern = re.compile("[零一二两三四五六七八九]")
        match = pattern.finditer(target)
        for m in match:
            target = pattern.sub(str(cls.wordToNumber(m.group())), target, 1)

        pattern = re.compile("(?<=(周|星期))[末天日]")
        match = pattern.finditer(target)
        for m in match:
            target = pattern.sub(str(cls.wordToNumber(m.group())), target, 1)

        pattern = re.compile("(?<!(周|星期))0?[0-9]?十[0-9]?")
        match = pattern.finditer(target)
        for m in match:
            group = m.group()
            s = group.split("十")
            num = 0
            ten = cls.strToInt(s[0])
            if ten == 0:
                ten = 1
            unit = cls.strToInt(s[1])
            num = ten * 10 + unit
            target = pattern.sub(str(num), target, 1)

        pattern = re.compile("0?[1-9]百[0-9]?[0-9]?")
        match = pattern.finditer(target)
        for m in match:
            group = m.group()
            s = group.split("百")
            s = list(filter(None, s))
            num = 0
            if len(s) == 1:
                hundred = int(s[0])
                num += hundred * 100
            elif len(s) == 2:
                hundred = int(s[0])
                num += hundred * 100
                num += int(s[1])
            target = pattern.sub(str(num), target, 1)

        pattern = re.compile("0?[1-9]千[0-9]?[0-9]?[0-9]?")
        match = pattern.finditer(target)
        for m in match:
            group = m.group()
            s = group.split("千")
            s = list(filter(None, s))
            num = 0
            if len(s) == 1:
                thousand = int(s[0])
                num += thousand * 1000
            elif len(s) == 2:
                thousand = int(s[0])
                num += thousand * 1000
                num += int(s[1])
            target = pattern.sub(str(num), target, 1)

        pattern = re.compile("[0-9]+万[0-9]?[0-9]?[0-9]?[0-9]?")
        match = pattern.finditer(target)
        for m in match:
            group = m.group()
            s = group.split("万")
            s = list(filter(None, s))
            num = 0
            if len(s) == 1:
                tenthousand = int(s[0])
                num += tenthousand * 10000
            elif len(s) == 2:
                tenthousand = int(s[0])
                num += tenthousand * 10000
                num += int(s[1])
            target = pattern.sub(str(num), target, 1)

        return target

    @classmethod
    def specialTranslator(cls, s):
        s = s.replace("十一国庆节", "国庆节")
        return s



    @classmethod
    def wordToNumber(cls, s):
        """
        方法numberTranslator的辅助方法，可将[零-九]正确翻译为[0-9]
        :param s: 大写数字
        :return: 对应的整形数，如果不是数字返回-1
        """
        if (s == '零') or (s == '0'):
            return 0
        elif (s == '一') or (s == '1'):
            return 1
        elif (s == '二') or (s == '两') or (s == '2'):
            return 2
        elif (s == '三') or (s == '3'):
            return 3
        elif (s == '四') or (s == '4'):
            return 4
        elif (s == '五') or (s == '5'):
            return 5
        elif (s == '六') or (s == '6'):
            return 6
        elif (s == '七') or (s == '天') or (s == '日') or (s == '末') or (s == '7'):
            return 7
        elif (s == '八') or (s == '8'):
            return 8
        elif (s == '九') or (s == '9'):
            return 9
        else:
            return -1

    @classmethod
    def strToInt(cls, s):
        try:
            res = int(s)
        except:
            res = 0
        return res


import re
import json
import requests
import asyncio
import collections
from datetime import datetime
from smart_tag_tools.TimeTagHelper.TimeHandle import handle
from smart_tag_tools.LinqTool import linq
from pypinyin import lazy_pinyin, Style


class SmartTagsConvertHelper:
    """
    智能标签处理
    """

    def __init__(self, kwargs):
        self.kwargs = kwargs
        # 知识库服务健康检查状态
        self.kb_interface_status = self.__kb_heartbeat_detection__()
        # 实体转换关系记录
        self.entity_convert_map = {'NAME': {}, 'LOC': {}, 'ORG': {}, 'TIME': {}, 'JOB': {}}
        # 实体类型与操作记录类型映射
        self.entity_type_map = {'people': 'NAME', 'loc': 'LOC', 'org': 'ORG', 'time': 'TIME'}

    def __kb_heartbeat_detection__(self):
        """
        知识库心跳检测
        :return:
        """
        try:
            res = requests.get(url='http://{}:{}{}/health-check'.format(self.kwargs['kb_service']['ip'],
                                                                        self.kwargs['kb_service']['port'],
                                                                        self.kwargs['kb_service']['url_prefix']),
                               timeout=self.kwargs['kb_service']['timeout'])
            if res.status_code == 200:
                return True
            else:
                return False
        except:
            return False

    def GetDateFromText(self, date_str):
        """
        解析字符串中包含的日期数据，通常用于从标题中提取首播日期
        第一现场191025//2019-09-28 吉林新闻联播
        :param date_str: 字符串
        :return:
        """
        if not isinstance(date_str, str):
            raise TypeError('参数类型错误')
        if date_str:
            for reg in self.kwargs['date_reg']:
                date_list = re.findall(reg, date_str, re.S)
                if date_list:
                    year, month, day = date_list[0]
                    if len(year) == 2:
                        year = str(datetime.now().year)[:2] + str(year)
                    try:
                        result = datetime(year=int(year), month=int(month), day=int(day)).strftime('%Y-%m-%d')
                    except:
                        result = ''
                    return result

    def GetNewsDate(self, date_list, reference_date, is_callback=False):
        """
        根据NLP从语音、文稿、字幕中提取出的时间词，结合首播日期等信息，
        获得符合标准的日期标签（YYYYMM或YYYYMMDD)
        2020年1月7日 相关日期：从正面筛选，只需要显示 年月日（号），月日（号），日（号）
        """
        if not isinstance(date_list, list):
            raise TypeError('参数类型错误')
        items = handle(date_list, reference_date)
        result = dict(zip(date_list, items))
        if is_callback:
            self.CallFun(result, 'TIME')
        return list(filter(lambda x: x, result.values()))

    @staticmethod
    def __abs_(value):
        now = datetime.now()
        if value <= now:
            value = 1 / (value - now).total_seconds()
        else:
            value = (value - now).total_seconds()
        return value

    def GetTimeByOrder(self, by_sort_list, order):
        """
        时间排序
        1. 由近到远，先过去，后未来排列；
        2. 按时间从小到大排列；
        3. 按时间从大到小排列。
        :param by_sort_list:待排序的时间列表
        :param order:排序方式
        :return:排序后的时间列表
        """
        if not isinstance(order, int):
            raise TypeError('传入的类型错误,整型数据被需要')
        sorted_list = list()
        if by_sort_list:
            switch = {
                1: lambda x: sorted(x, key=self.__abs_, reverse=False),
                2: lambda x: sorted(x, reverse=False),
                3: lambda x: sorted(x, reverse=True)
            }
            sorted_list = switch.get(order, 1)(by_sort_list)
        return sorted_list

    def GetDataByPriority(self, kwargs):
        """
        按优先级获取
        对于可存在于多个来源的数据，可按预设优先级获取数据，找到有效数据则返回。
        :param kwargs:
        :return:
        """
        if not isinstance(kwargs, dict):
            raise TypeError('参数类型错误,字典被需要')
        for k in sorted(kwargs):
            if kwargs[k]:
                return kwargs[k]

    def GetUnion(self, *args):
        """
        多个标签集合取并集后返回
        :param args:
        :return:
        """
        union_data = set()
        for data in args:
            union_data.update(data)
        return union_data

    def GetIntersection(self, *args):
        """
        多个标签集合取交集后返回
        :param args:
        :return:
        """
        intersection_data = args[0]
        for data in args:
            intersection_data.intersection_update(data)
        return intersection_data

    def __dict_to_obj__(self, dic):
        """
        字典转为json对象，实现类.属性的操作
        :param dic:
        :return:
        """
        if not isinstance(dic, dict):
            raise TypeError('传入的数据类型错误,字典被需要')

        class JSONObject:
            def __init__(self, d):
                self.__dict__ = d

        return json.loads(json.dumps(dic, ensure_ascii=False), object_hook=JSONObject, encoding='utf-8')

    def __kb_interface_call(self, route, data):
        """
        知识库接口调用
        :param route: 相对路径
        :param data: json数据
        :return:
        """
        try:
            res = requests.post(url='http://{}:{}{}{}'.format(self.kwargs['kb_service']['ip'],
                                                              self.kwargs['kb_service']['port'],
                                                              self.kwargs['kb_service']['url_prefix'], route),
                                json=data,
                                timeout=self.kwargs['kb_service']['timeout']).json()
            if res['success']:
                return res['data']
        except:
            return None

    def GetColumnFromText(self, text, channel=None):
        """
        栏目提取
        解析字符串中包含的栏目数据，通常用于从标题中提取栏目
        :param text:文本字符串
        :param channel:频道名称
        :return:
        """
        data = {
            'text': text,
            'channel': channel
        }
        if not self.kb_interface_status:
            return None
        res = self.__kb_interface_call('/v1/label/column-names', data)
        if res:
            result = res['result']
            if result:
                return result[0]['column']
        else:
            return None

    def GetNewsLocation(self, address, is_callback=False):
        """
        地点筛选
        根据NLP从语音、文稿、字幕中提取出的地点词, 经过清洗筛选，过滤掉过大，过小的地点词，返回有意义的地点词（具体算法待定）。 过滤掉类似 “四楼咖啡厅"，"客厅"这类词语
        """
        if not isinstance(address, list):
            raise TypeError('参数类型错误')
        result = dict()
        if not self.kb_interface_status:
            return address
        res = self.__kb_interface_call('/v1/label/filtered-locations', address)
        if res:
            for item in res:
                result[item['originvalue']] = item['result'][0].get('name', '') if item['result'] else ''
            if is_callback:
                self.CallFun(result, 'LOC')
            return list(filter(lambda x: x, result.values()))
        return address

    def GetRelatedDivision(self, address):
        """
        获得相关行政区
        根据具体地点返回所属行政区划（完整路径）
        :param address: 地点
        :return:
        """
        if not isinstance(address, list):
            raise TypeError('参数类型错误')
        result = dict()
        if not self.kb_interface_status:
            return result
        res = self.__kb_interface_call('v1/label/related-divisions', address)
        if res:
            for item in res:
                result[item['originvalue']] = item.get('result', '')
        return result

    def GetFullOrgName(self, agency: list, is_callback=False):
        """
        机构名补全
        对于AI提取的不完整的标签数据，补充限定词。限定词依据电视台所属行政区划数据获得。
        补充地区限定词，当机构名称首字为“省”时，自动补齐“XX省”，如“省人民政府”补充为“XX省人民政府；
        """
        result = dict([(x, x) for x in agency])
        if agency:
            type_dict = collections.OrderedDict((key, value) for key, value in self.kwargs['tv_area'].items() if value)
            for index, item in enumerate(agency):
                if list(filter(lambda x: True if item.startswith(x) or item.startswith('州') else False,
                               type_dict.keys())):
                    if '自治州' not in item and '州' in item:
                        item = item.replace('州', '自治州')
                    for k, v in type_dict.items():
                        if k in item:
                            if k == '区' and '自治区' in item:
                                continue
                            result[agency[index]] = type_dict.get(k).replace(k, '') + item
            if is_callback:
                self.CallFun(result, 'ORG')
        return list(filter(lambda x: x, result.values()))

    def GetNormalizedOrgName(self, org, is_callback=False):
        """
        机构名归一化
        按照机构库名字进行归一化处理，将简称转换为全称
        """
        if not isinstance(org, list):
            raise TypeError('参数类型错误')
        result = dict()
        if not self.kb_interface_status:
            return org
        res = self.__kb_interface_call('/v1/label/normalized-names?type=org', org)
        if res:
            for item in res:
                result[item['origin']] = item.get('result', '')
            if is_callback:
                self.CallFun(result, 'ORG')
            return list(filter(lambda x: x, result.values()))
        return org

    def GetAddressByOrder(self, address, order=1):
        """
        地点排序
        将无序的地点标签数组按指定方式排列，可选以下两种方式：
        1. 按地点从大到小，由本地到外地排列；
        2. 按地点音序排列
        排序方式 level|pronunciation 大小，音序
        :param address: 排序列表
        :param order: 类型
        :return:
        """
        __map__ = {1: 'level', 2: 'pronunciation'}
        if not isinstance(address, list):
            raise TypeError('参数类型错误')
        if not self.kb_interface_status:
            return address
        data = {
            "originvalue": address,
            "pattern": __map__.get(int(order), 1)
        }
        res = self.__kb_interface_call('/v1/label/sorted-locationnames', data)
        if res:
            return res['result']
        return address

    def GetOrgTagsByOrder(self, org, order=1):
        """
        机构标签排序
        将机构标签按由大到小，由本地到外地顺序排列
        :param org: 排序列表
        :param order: 类型
        :return:
        """
        __map__ = {1: 'level'}
        if not isinstance(org, list):
            raise TypeError('参数类型错误')
        if not self.kb_interface_status:
            return org
        data = {
            'originvalue': org,
            'pattern': __map__.get(int(order))
        }
        res = self.__kb_interface_call('/v1/label/sorted-orgnames', data)
        if res:
            return res['result']
        return org

    def GetAlternative(self, replace):
        """
        推荐替换词
        主要用在人工审校环节，按以下逻辑处理：
        1. 标准叙词库中的词语，则直接使用，不推荐修改；
        2. 标准叙词表中不存在的词语，但是在别名字段中存在的词语，推荐修改为实体本名；
        3. 标准叙词表的本名、别名均不存在的，通过近似算法找最相近的5个进行推荐。
        :param replace:被替换词列表
        :return:替换后的列表
        """
        if not isinstance(replace, list):
            raise TypeError('参数类型错误')
        result = dict()
        if not self.kb_interface_status:
            return replace
        res = self.__kb_interface_call('/v1/label/alternative-words', replace)
        if res:
            for item in res:
                result[item['originvalue']] = item['result'][0].get('name', '') if item['result'] else ''
            return result
        return replace

    @staticmethod
    def NumberOfStatistical(items):
        """
        统计词条出现的次数
        :param items:
        :return:
        """
        if not isinstance(items, list):
            raise TypeError('参数类型错误')
        # noinspection PyArgumentList
        return collections.Counter(items)

    def JudgeIsColumn(self, items):
        """
        判断是不是栏目
        :param items:
        :return:
        """
        if not isinstance(items, list):
            raise TypeError('参数类型错误')
        result = list()
        if not self.kb_interface_status:
            return result
        res = self.__kb_interface_call('/v1/label/judged-column-names', items)
        if res:
            result = [tmp['originvalue'] for tmp in filter(lambda x: True if x.get('result') else False, res)]
        return result

    def JudgeIsLocName(self, items):
        """
        判断是不是地区名词
        :param items:
        :return:
        """
        if not isinstance(items, list):
            raise TypeError('参数类型错误')
        result = list()
        if not self.kb_interface_status:
            return result
        res = self.__kb_interface_call('/v1/label/judged-loc-names', items)
        if res:
            result = [tmp['originvalue'] for tmp in filter(lambda x: True if x.get('result') else False, res)]
        return result

    def GetSpecificIdentityPeople(self, people):
        """
        获取特定身份的人
        用于设别主持人，记者或其他人物身份
        :param people:
        :return:
        """
        if not isinstance(people, list):
            raise TypeError('参数类型错误')
        result = list()
        if not self.kb_interface_status:
            return result
        res = self.__kb_interface_call('/v1/label/reporters-comperes', people)
        if res:
            return res
        return result

    def NamesOfErrorCorrection(self, people, is_callback=False):
        """
        人名纠错处理（目前只考虑带头衔的人名）
        """
        if not isinstance(people, list):
            raise TypeError('参数类型错误')
        if not self.kb_interface_status:
            return people
        res = self.__kb_interface_call('/v1/label/homophones', people)
        if res:
            if is_callback:
                self.CallFun(dict(linq(res).select(lambda x: (x.get('origin', ''), x.get('result'))).tolist()), 'NAME')
            return linq(res).select(
                lambda x: {'id': x.get('id'), 'name': x.get('result'), 'confidence': x.get('confidence')}).tolist()
        return people

    def GetNormalizedHumanName(self, people, is_callback=False):
        """
        相关人名归一化
        按照人物库名字进行归一化处理，将常用的多种称谓进行统一，对音译人名进行统一。
        """
        if not isinstance(people, list):
            raise TypeError('参数类型错误')
        if not self.kb_interface_status:
            return people
        res = self.__kb_interface_call('/v1/label/normalized-names?type=people', people)
        if res:
            if is_callback:
                self.CallFun(dict(linq(res).select(lambda x: (x.get('origin', ''), x.get('result'))).tolist()), 'NAME')
            return linq(res).select(lambda x: {'id': x.get('id'), 'name': x.get('result')}).tolist()
        return people

    def GetPeopleTagsByOrder(self, people, order):
        """
        人名标签排序
        将无序的人名标签数组按指定方式排列，可选以下3种方式：
        1. 按照人物级别排序（元首-中央政治局常委【有序】-中央政治局委员-中央委员会委员-前任总书记-曾担任政治局常委的党和国家领导人-省委书记-省长-省委常委【有序】-省厅局级领导-市委书记-市长-市委常委【有序】-市委办局级领导-区县书记-区县长。如同等级别则先主后宾排列。
        2. 按姓名音序排列
        3. 按姓名笔画排列
        排序方式 level | pronunciation | stroke 级别，音序，笔画数
        :param people: 排序列表
        :param order 类型
        :return:
        """
        __map__ = {1: 'level', 2: 'pronunciation', 3: 'stroke'}
        if not isinstance(people, list):
            raise TypeError('参数类型错误')
        if not self.kb_interface_status:
            return people
        data = {
            'originvalue': people,
            'pattern': __map__.get(int(order), 1)
        }
        res = self.__kb_interface_call('/v1/label/sorted-humannames', data)
        if res:
            return res['result']
        return people

    def JudgeIsSensitive(self, items):
        """
        判断是不是敏感人物
        :param items:
        :return:
        """
        if not isinstance(items, list):
            raise TypeError('参数类型错误')
        if not self.kb_interface_status:
            return items
        result = list()
        res = self.__kb_interface_call('/v1/label/judged-sensitive-peoples', items)
        if res:
            for tmp in res:
                if tmp.get('result'):
                    tmp['name'] = '(敏感人物)' + tmp['name']
                result.append(tmp)
            return result
        return items

    @staticmethod
    def edit(str1, str2):
        """
        算两个字符串的最小编辑距离
        """
        matrix = [[i + j for j in range(len(str2) + 1)]
                  for i in range(len(str1) + 1)]

        for i in range(1, len(str1) + 1):
            for j in range(1, len(str2) + 1):
                if str1[i - 1] == str2[j - 1]:
                    d = 0
                else:
                    d = 1
                matrix[i][j] = min(matrix[i - 1][j] + 1, matrix[i][j - 1] + 1,
                                   matrix[i - 1][j - 1] + d)

        return round(1 - (matrix[len(str1)][len(str2)] / len(str2)), 2)

    def PinyinIsSimilarity(self, x, y):
        """
        判断两个词拼音是不是相似
        """
        pinyin_x = ''.join(lazy_pinyin(x.replace('(敏感人物)', '').split('(')[0], style=Style.NORMAL))
        pinyin_y = ''.join(lazy_pinyin(y.replace('(敏感人物)', '').split('(')[0], style=Style.NORMAL))
        return self.edit(pinyin_x, pinyin_y) > self.kwargs.get('PinyinSimilarity', 0.8)

    async def single_tag_correct(self, word, typ, tag_map):
        data = linq(self.kwargs.get('TagCorrectRecord', {}).get(typ, [])).where(
            lambda x: x['before_correction'] == word).order_by(
            lambda x: x['operate_times'], default=True).first()
        if data:
            tag_map[word] = data['after_correction']

    def BatchTagCorrect(self, entity_string, typ, is_callback=False):
        """
        批量标签修正
        """
        if not entity_string:
            return ''
        tag_map = {k: k for k in entity_string.split(',')}
        new_loop = asyncio.new_event_loop()
        asyncio.set_event_loop(new_loop)
        loop = asyncio.get_event_loop()
        tasks = [self.single_tag_correct(tag, typ, tag_map) for tag in tag_map.keys()]
        loop.run_until_complete(asyncio.wait(tasks))
        if is_callback:
            self.CallFun(tag_map, self.entity_type_map[typ])
        return ','.join(tag_map.values())

    @staticmethod
    def chunk_merge(chunk, typ):
        result = list()
        for name, array in linq(chunk).group_by('name').items():
            if len(array) > 1:
                time_line = []
                dic = {'name': name}
                if typ == 'NAME':
                    dic['ids'] = ','.join(linq([arr['ids'] for arr in array]).where(lambda x: x).distinct().tolist())
                for arr in array:
                    time_line.extend(arr['timeline'])
                dic['timeline'] = linq(time_line).order_by(lambda x: (x['source'], x['inpoint'])).distinct().tolist()
                result.append(dic)
            else:
                result.extend(array)
        return result

    def EntityBackFlow(self, entity):
        """
        扫描实体元数据，回流更新记录
        这里需要对相同实体合并时间线
        """
        for group in entity:
            kind = group['type']
            for dic in group['entitydata']:
                name = self.entity_convert_map[kind].get(dic['name'])
                if name:
                    dic['name'] = name
            group['entitydata'] = self.chunk_merge(group['entitydata'], group['type'])

    @staticmethod
    def title_pipeline(x):
        return x.replace('(敏感人物)', '').split('(')[0]

    def CallFun(self, relation, kind):
        """
        回调函数，用于记录实体转换关系
        """
        # 人名实体更新，需要忽略头衔
        if kind == 'NAME':
            relation = dict(linq(relation.items()).select(
                lambda x: (self.title_pipeline(x[0]), self.title_pipeline(x[1]))).tolist())
        # 将每一步获取的转换关系映射更新到实体关系记录表
        for k, v in relation.items():
            key_array = linq(self.entity_convert_map[kind].items()).where(lambda x: x[1] == k).select(
                lambda x: x[0]).tolist()
            if key_array:
                self.entity_convert_map[kind].update({key: v for key in key_array})
            else:
                self.entity_convert_map[kind][k] = v



import codecs
import os
import setuptools

try:
    from setuptools import setup
except:
    from distutils.core import setup


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


platforms = ['linux/Windows']
classifiers = [
    'Development Status :: 3',
    'Topic :: Text Processing',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
]

install_requires = [
    # 'numpy>=1.11.1',
    # 'pandas>=0.19.0'
    'sxtwl==1.0.7',
    'requests>=2.22.0',
    'python-dateutil==2.8.0',
    'arrow==0.13.1',
    'regex==2020.2.20',
    'pypinyin==0.36.0'
]

setup(name='smart_tag_tools',
      version='0.0.4',
      description='A test module for caozhi',
      long_description='口语化时间模块解析；代码优化',
      packages=setuptools.find_packages(),
      include_package_data=True,
      author="caozhi",
      author_email="hjp46193816@163.com",
      url="https://benyo.github.io",
      license="Apache License, Version 2.0",
      platforms=platforms,
      install_requires=install_requires,
      zip_safe=False
      )



#!/usr/bin/python3
# -*- coding:utf-8 -*-
from datetime import datetime, timedelta
import requests
import base64
import hmac
from hashlib import sha1


class HiveSearchApi:
    """
    hive查询类
    """
    system_secret_key = "e2c84557fc9c4d069bfc7b958abccdca"
    user_code = "admin"
    site_code = operate_site = "S1"
    system_code = "MAM"
    contenttype = 'application/json'
    GMT_FORMAT = '%a, %d %b %Y %H:%M:%S GMT'

    def __init__(self, ip, port, timeout):
        self.url = "http://{}:{}/sobeyhive-bp/v1/entity".format(ip, port)
        self.timeout = timeout

    def get_entityinfo_url(self, contentid):
        data = {'contentid': contentid, 'pathtype': 1}
        dateStr = (datetime.utcnow() - timedelta(minutes=0)).strftime(self.GMT_FORMAT)
        sign = self.__get_request_signature_header('GET', dateStr, self.contenttype)
        headers = {
            'sobeyhive-http-site': self.site_code,
            'sobeyhive-http-date': dateStr,
            'sobeyhive-http-authorization': sign,
            'sobeyhive-http-system': self.system_code,
            'sobeyhive-http-operate-site': self.operate_site,
            'Content-Type': self.contenttype,
            'current-user-code': self.user_code
        }
        res = requests.get(url=self.url, headers=headers, params=data, timeout=self.timeout)
        return res.json()

    def __get_request_signature_header(self, method, dateStr, contenttype='application/json'):
        sign = self.__get_request_signature(
            method, '', self.system_secret_key, dateStr, contenttype)

        return 'SobeyHive {system_code}:{sign}'.format(system_code=self.system_code, sign=sign)

    @staticmethod
    def __get_request_signature(method, conentmd5, encryptKey, dateStr, contenttype='application/json'):
        stringToSign = '{method}\n{conentmd5}\n{contenttype}\n{dateStr}'.format(method=method,
                                                                                conentmd5=conentmd5,
                                                                                contenttype=contenttype,
                                                                                dateStr=dateStr)

        hmac_code = hmac.new(encryptKey.encode(), stringToSign.encode(), sha1)
        signature = base64.b64encode(hmac_code.digest())
        return str(signature, 'utf8')
